<?php
/**
 * Plugin updater functions.
 *
 * @package CL_plugin_updater
 */

class CL_Plugin_Updater {
    function __construct() {

        $this->option_key       = 'warrior_edd_salesbooster_setting_key';
		$this->option_status    = 'warrior_edd_salesbooster_setting_status';
		$this->remote_api_url   = 'https://cepatlakoo.com';
        // $this->remote_api_url   = 'http://wp.loc';
        $this->version          = '1.0.0';
        // $this->item_name        = 'EDD Sales Notification';
		$this->item_name        = 'Cepatlakoo';
		$this->item_id          = 126;
        // $this->item_id          = 5202;
        $this->author           = 'Themewarrior';
        $this->plugin_page      = 'edit.php?post_type=download&page=warrior_sales_notification';
        $this->beta             = false;
        $this->post_active      = 'warrior_edd_booster_license_activate';
        $this->post_deactive    = 'warrior_edd_booster_license_deactivate';
        $this->post_key         = 'warrior_edd_salesbooster_setting_key';
        $this->renew_url        = '';
        $this->strings = array(
            'plugin-license'            => __( 'Plugin License', 'edd-sales-notification' ),
            'enter-key'                 => __( 'Enter your plugin license key.', 'edd-sales-notification' ),
            'license-key'               => __( 'License Key', 'edd-sales-notification' ),
            'license-action'            => __( 'License Action', 'edd-sales-notification' ),
            'deactivate-license'        => __( 'Deactivate License', 'edd-sales-notification' ),
            'activate-license'          => __( 'Activate License', 'edd-sales-notification' ),
            'status-unknown'            => __( 'License status is unknown.', 'edd-sales-notification' ),
            'renew'                     => __( 'Renew?', 'edd-sales-notification' ),
            'unlimited'                 => __( 'unlimited', 'edd-sales-notification' ),
            'license-key-is-active'     => __( 'License key is active.', 'edd-sales-notification' ),
            'expires%s'                 => __( 'Expires %s.', 'edd-sales-notification' ),
            'expires-never'             => __( 'Lifetime License.', 'edd-sales-notification' ),
            '%1$s/%2$-sites'            => __( 'You have <span class="active-sites">%1$s / %2$s</span> sites activated.', 'edd-sales-notification' ),
            'license-key-expired-%s'    => __( 'License key expired %s.', 'edd-sales-notification' ),
            'license-key-expired'       => __( 'License key has expired.', 'edd-sales-notification' ),
            'license-keys-do-not-match' => __( 'License keys do not match.', 'edd-sales-notification' ),
            'license-is-inactive'       => __( 'License is inactive.', 'edd-sales-notification' ),
            'license-key-is-disabled'   => __( 'License key is disabled.', 'edd-sales-notification' ),
            'site-is-inactive'          => __( 'Site is inactive.', 'edd-sales-notification' ),
            'license-status-unknown'    => __( 'License status is unknown.', 'edd-sales-notification' ),
            'cl-sites-limited'    		=> __( 'Your license key has reached domain activation limit.', 'edd-sales-notification' ),
        );

		add_action( 'admin_init', array( $this, 'edd_init' ) );
		add_action( 'admin_init', array( $this, 'edd_action' ) );
		add_action( 'update_option_'.$this->option_key, array( $this, 'edd_activate' ), 10, 2 );
		add_action( 'admin_notices', array( $this, 'edd_admin_notices' ) );
    }

    public function edd_init() {
        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }

        if( !class_exists( 'EDD_SL_Plugin_Updater' ) ) {
            // load our custom updater
            include( dirname( __FILE__ ) . '/EDD_SL_Plugin_Updater.php' );
        }

        $license_key = trim( get_option( 'warrior_edd_salesbooster_setting_key' ) );

        $edd_updater = new EDD_SL_Plugin_Updater( $this->remote_api_url, __FILE__, array(
            'version'   => $this->version,		// current version number
            'license'   => $license_key,	// license key (used get_option above to retrieve from DB)
            'item_id'   => $this->item_id,	// id of this plugin
            'author'    => $this->author,	// author of this plugin
            'url'       => home_url(),
            'beta'      => $this->beta // set to true if you wish customers to receive update notifications of beta releases
        ) );
    }

    public function edd_action() {
        if( isset( $_POST[$this->post_active] ) ) {
            $this->edd_activate();
        }
        elseif( isset( $_POST[$this->post_deactive] ) ) {
            $this->edd_deactivate();
        }
    }
    
    public function edd_activate() {
        $license = trim( get_option( $this->option_key ) );
        if( !empty($_POST[$this->post_key]) && $_POST[$this->post_key] != $license ){
            $license = $_POST[$this->post_key];
            update_option( $this->option_key, $_POST[$this->post_key] );
        }

		// Data to send in our API request.
		$api_params = array(
			'edd_action' => 'activate_license',
			'license'    => $license,
			'item_id'    => $this->item_id,
			'url'        => home_url()
		);

        $response = $this->get_api_response( $api_params );
        // var_dump($response); exit();
		// make sure the response came back okay
		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

			if ( is_wp_error( $response ) ) {
				$message = $response->get_error_message();
			} else {
				$message = __( 'An error occurred, please try again.' );
			}

		} else {

			$license_data = json_decode( wp_remote_retrieve_body( $response ) );

			if ( false === $license_data->success ) {

				switch( $license_data->error ) {

					case 'expired' :

						$message = sprintf(
							__( 'Your license key expired on %s.' ),
							date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
						);
						break;

					case 'revoked' :

						$message = __( 'Your license key has been disabled.' );
						break;

					case 'missing' :

						$message = __( 'Invalid license.' );
						break;

					case 'invalid' :
					case 'site_inactive' :

						$message = __( 'Your license is not active for this URL.' );
						break;

					case 'item_name_mismatch' :

						$message = sprintf( __( 'This appears to be an invalid license key for %s.' ), $args['name'] );
						break;

					case 'no_activations_left':

						$message = __( 'Your license key has reached its activation limit.' );
						break;

					case 'cl-sites-limited':

						$message = __( 'Your license key has reached domain activation limit.' );
						break;

					default :

						$message = __( 'An error occurred, please try again.' );
						break;
				}

				if ( ! empty( $message ) ) {
					$base_url = admin_url( $this->plugin_page );
					$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );

					wp_redirect( $redirect );
					exit();
				}

			}

		}

		// $response->license will be either "active" or "inactive"
		if ( $license_data && isset( $license_data->license ) ) {
			update_option( $this->option_status, $license_data );
			delete_transient( $this->option_key );
        }
        
        $base_url = admin_url( $this->plugin_page );
        if ( ! empty( $message ) ) {
			$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );
        }
        else{
            $redirect = $base_url;
        }
        
        wp_redirect( $redirect );
		exit();
    }

    public function edd_deactivate() {

		// Retrieve the license from the database.
		$license = trim( get_option( $this->option_key ) );

		// Data to send in our API request.
		$api_params = array(
			'edd_action' => 'deactivate_license',
			'license'    => $license,
			'item_name'  => urlencode( $this->item_name ),
			'url'        => home_url()
		);

		$response = $this->get_api_response( $api_params );

		// make sure the response came back okay
		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

			if ( is_wp_error( $response ) ) {
				$message = $response->get_error_message();
			} else {
				$message = __( 'An error occurred, please try again.' );
			}

		} else {

			$license_data = json_decode( wp_remote_retrieve_body( $response ) );

			// $license_data->license will be either "deactivated" or "failed"
			if ( $license_data && ( $license_data->license == 'deactivated' ) ) {
				delete_option( $this->option_key );
				delete_option( $this->option_status );
				delete_transient( $this->option_key );
			}

		}

		if ( ! empty( $message ) ) {
			$base_url = admin_url( $this->plugin_page );
			$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );

			wp_redirect( $redirect );
			exit();
		}

		wp_redirect( admin_url( $this->plugin_page ) );
		exit();

	}
    
    public function edd_sample_register_option() {
        // creates our settings in the options table
        register_setting('edd_sample_license', 'warrior_edd_salesbooster_setting_key', 'edd_sanitize_license' );
    }
    
    public function edd_sanitize_license( $new ) {
        $old = trim( get_option( $this->option_key ) );
        if( $old && $old != $new ) {
            delete_option( $this->option_key ); // new license has been entered, so must reactivate
            delete_option( $this->option_status ); // new license has been entered, so must reactivate
        }
        return $new;
    }

    function get_api_response( $api_params ) {

        // Call the custom API.
        $response = wp_remote_post( $this->remote_api_url, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );
        
        // Make sure the response came back okay.
        if ( is_wp_error( $response ) ) {
            wp_die( $response->get_error_message(), __( 'Error' ) . $response->get_error_code() );
        }

        return $response;
    }
     
    public function edd_check_license( $raw = false ) {

		$license = trim( get_option( $this->option_key ) );
		$strings = $this->strings;
		
		if( !empty($license) ){
			$api_params = array(
				'edd_action' => 'check_license',
				'license'    => $license,
				'item_name'  => urlencode( $this->item_name ),
				'url'        => home_url()
            );
            
			if( ! get_transient( $this->option_key, false ) ){
				$response = $this->get_api_response( $api_params );
				set_transient( $this->option_key, $response, ( 60 * 60 * 24 ) );
			}
			else{
				$response = get_transient( $this->option_key );
			}

			// make sure the response came back okay
			if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

				if ( is_wp_error( $response ) ) {
					$message = $response->get_error_message();
				} else {
					$message = $strings['license-status-unknown'];
				}
				delete_option( $this->option_status );
				delete_transient( $this->option_key );
			} else {
				
				// var_dump( wp_remote_retrieve_body( $response ) );
				$license_data = json_decode( wp_remote_retrieve_body( $response ) );
				
				// If response doesn't include license data, return
				if ( !isset( $license_data->license ) ) {
					$message = $strings['license-status-unknown'];
					update_option( $this->option_status, $license_data );
					delete_transient( $this->option_key );
					return $message;
				}

				// We need to update the license status at the same time the message is updated
				if ( $license_data && isset( $license_data->license ) ) {
					update_option( $this->option_status, $license_data );
					delete_transient( $this->option_key );
				}

				// Get expire date
				$expires = false;
				if ( isset( $license_data->expires ) && 'lifetime' != $license_data->expires ) {
					$expires = date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) );
					$renew_link = '<a href="' . esc_url( $this->get_renewal_link() ) . '" target="_blank">' . $strings['renew'] . '</a>';
				} elseif ( isset( $license_data->expires ) && 'lifetime' == $license_data->expires ) {
					$expires = 'lifetime';
				}

				// Get site counts
				$site_count = $license_data->site_count;
				$license_limit = $license_data->license_limit;

				// If unlimited
				if ( 0 == $license_limit ) {
					$license_limit = $strings['unlimited'];
				}

				if ( $license_data->license == 'valid' ) {
					$message = $strings['license-key-is-active'] . ' ';
					if ( isset( $expires ) && 'lifetime' != $expires ) {
						$message .= sprintf( $strings['expires%s'], $expires ) . ' ';
					}
					if ( isset( $expires ) && 'lifetime' == $expires ) {
						$message .= $strings['expires-never'];
					}
					if ( $site_count && $license_limit ) {
						$message .= sprintf( $strings['%1$s/%2$-sites'], $site_count, $license_limit );
					}
				} else if ( $license_data->license == 'expired' ) {
					if ( $expires ) {
						$message = sprintf( $strings['license-key-expired-%s'], $expires );
					} else {
						$message = $strings['license-key-expired'];
					}
					if ( $renew_link ) {
						$message .= ' ' . $renew_link;
					}
				} else if ( $license_data->license == 'invalid' || $license_data->license == 'item_name_mismatch') {
					$message = $strings['license-keys-do-not-match'];
				} else if ( $license_data->license == 'inactive' ) {
					$message = $strings['license-is-inactive'];
				} else if ( $license_data->license == 'disabled' ) {
					$message = $strings['license-key-is-disabled'];
				} else if ( $license_data->license == 'site_inactive' ) {
					// Site is inactive
					$message = $strings['site-is-inactive'];
				} else if ( $license_data->license == 'cl-sites-limited' ) {
					// Site is inactive
					$message = $strings['cl-sites-limited'];
				} else {
					$message = $strings['license-status-unknown'];
				}

			}
		}
		else{
			$license_data = (object)[];
			$license_data->license = 'empty';
			$message = $strings['enter-key'];
		}

		if( $raw && !empty($license_data ) ){
			return array('raw' => $license_data, 'msg' => $message);
		}
		else{
			return $message;
		}
    }
    
    public function edd_admin_notices() {
        $check = $this->edd_check_license(true);
        // var_dump($check); exit();
        if ( isset($check['raw']) && $check['raw']->license == 'empty' ) : ?>
            <div class="notice notice-error">
                <p><strong><?php echo $this->item_name; ?></strong>
                <br /> <?php echo $check['msg'] ?>
                <br /><a href="<?php echo admin_url($this->plugin_page); ?>" class="button button-primary"><?php esc_html_e( 'Activate License', 'edd-sales-notification' ); ?></a></p>
            </div>
        <?php elseif (  isset($check['raw']) && $check['raw']->license != 'valid' ) : ?>
            <div class="notice notice-error">
                <p><strong><?php echo $this->item_name; ?></strong>
                <br /> <?php echo $check['msg'] ?>
                <br /> <a href="<?php echo admin_url($this->plugin_page); ?>" class="button button-primary"><?php esc_html_e( 'Activate License', 'edd-sales-notification' ); ?></a></p>
            </div>
        <?php elseif( !is_array($check) ) : ?>
            <div class="notice notice-error">
                <p><strong><?php echo $this->item_name; ?></strong>
                <br /> <?php echo $check ?>
                <br /> <a href="<?php echo admin_url($this->plugin_page); ?>" class="button button-primary"><?php esc_html_e( 'Activate License', 'edd-sales-notification' ); ?></a></p>
            </div>
        <?php endif; 

        if ( isset( $_GET['sl_activation'] ) && ! empty( $_GET['message'] ) ) {
            switch( $_GET['sl_activation'] ) {
                case 'false':
                    $message = urldecode( $_GET['message'] );
                    ?>
                    <div class="error">
                        <p><?php echo $message; ?></p>
                    </div>
                    <?php
                    break;
                case 'true':
                default:
                    // Developers can put a custom success message here for when activation is successful if they way.
                    break;
            }
        }
    }

    function get_renewal_link() {
		// If a renewal link was passed in the config, use that
		if ( '' != $this->renew_url ) {
			return $this->renew_url;
		}

		// If download_id was passed in the config, a renewal link can be constructed
		$license_key = trim( get_option( $this->option_key , false ) );
		if ( '' != $this->item_id && $license_key ) {
			$url = esc_url( $this->remote_api_url );
			$url .= '/checkout/?edd_license_key=' . $license_key . '&download_id=' . $this->item_id;
			return $url;
		}

		// Otherwise return the remote_api_url
		return $this->remote_api_url;
	}
}